-- ============================================================================
-- === AutoLiftAxle.Lua
-- === Mod my [LSMT] Modding Team 
-- === LS25 /FS25
-- === Script by [LSMT] BaTt3RiE @ 2025
-- === Ver 1.0.0.0
-- ============================================================================

AutoLiftAxle = {}

function AutoLiftAxle.prerequisitesPresent(specializations)
    return true
end

function AutoLiftAxle.registerFunctions(vehicleType)
    SpecializationUtil.registerFunction(vehicleType, "setLiftAxleState", AutoLiftAxle.setLiftAxleState)
end

function AutoLiftAxle.registerEventListeners(vehicleType)
    SpecializationUtil.registerEventListener(vehicleType, "onLoad", AutoLiftAxle)
    SpecializationUtil.registerEventListener(vehicleType, "onPostLoad", AutoLiftAxle)
    SpecializationUtil.registerEventListener(vehicleType, "onUpdate", AutoLiftAxle)
    SpecializationUtil.registerEventListener(vehicleType, "onPostDetach", AutoLiftAxle)
    SpecializationUtil.registerEventListener(vehicleType, "onPostAttach", AutoLiftAxle)
    SpecializationUtil.registerEventListener(vehicleType, "onReadStream", AutoLiftAxle)
    SpecializationUtil.registerEventListener(vehicleType, "onWriteStream", AutoLiftAxle)
end

function AutoLiftAxle:onLoad(savegame)
    self.liftAxleActive = true
    self.liftAxleCheckTimer = 0
    self.liftAxleCheckInterval = 2000
    self.shopStateApplied = false

    self.liftAxleDesign = 3
    if self.configurations ~= nil and self.configurations["design"] ~= nil then
        self.liftAxleDesign = tonumber(self.configurations["design"]) or 3
    end
end

function AutoLiftAxle:onPostLoad(savegame)
    self:setLiftAxleState(self.liftAxleActive, false, true)
end

function AutoLiftAxle:onUpdate(dt)

    local isInShop = (g_gui.currentGuiName == "ShopConfigScreen")

    if isInShop and g_shopConfigScreen ~= nil then
        if g_shopConfigScreen.vehicle == self then
            self.liftAxleActive = true
            self.wasInShop = true

            if self.liftAxleDesign ~= 4 then
                if (self.liftAxleDesign == 1 or self.liftAxleDesign == 3)
                   and self:getAnimationExists("Animation_LiftAxle_Front") then
                    local direction = 1
                    self:playAnimation("Animation_LiftAxle_Front", direction, self:getAnimationTime("Animation_LiftAxle_Front"), true)
                end

                if (self.liftAxleDesign == 2 or self.liftAxleDesign == 3)
                   and self:getAnimationExists("Animation_LiftAxle_Back") then
                    local direction = 1
                    self:playAnimation("Animation_LiftAxle_Back", direction, self:getAnimationTime("Animation_LiftAxle_Back"), true)
                end
            end
            return
        end
    end

    if g_server then
        AutoLiftAxle.checkLiftAxleState(self, dt)
    end

end

function AutoLiftAxle.findMotorizedAttacherVehicle(vehicle)
    local currentVehicle = vehicle

    while currentVehicle ~= nil do
        if currentVehicle.getIsMotorStarted ~= nil then
            return currentVehicle
        end
        currentVehicle = currentVehicle:getAttacherVehicle()
    end

    return nil
end

function AutoLiftAxle.checkLiftAxleState(self, dt)
    local motorizedVehicle = AutoLiftAxle.findMotorizedAttacherVehicle(self)

    if motorizedVehicle == nil then
        self:setLiftAxleState(true)
        return
    end

    local motorOn = motorizedVehicle:getIsMotorStarted()
    if motorOn then
        self.liftAxleCheckTimer = self.liftAxleCheckTimer + dt
        if self.liftAxleCheckTimer >= self.liftAxleCheckInterval then
            self.liftAxleCheckTimer = 0



            local totalLoad = 0
            if self.spec_TrailerWeight.liftAxleThreshold then
                totalLoad = self.spec_TrailerWeight.currentWeight
            else
                if self.spec_fillUnits and self.spec_fillUnits.fillUnits then
                    for i, _ in ipairs(self.spec_fillUnits.fillUnits) do
                        totalLoad = totalLoad + self:getFillUnitFillLevel(i)
                    end
                else
                    totalLoad = self:getFillUnitFillLevel(1)
                end
            end

            if totalLoad > self.spec_TrailerWeight.liftAxleThreshold then
                self:setLiftAxleState(true)
            else
                self:setLiftAxleState(false)
            end
        end
    else
        self:setLiftAxleState(true)
    end
end


function AutoLiftAxle:onPostDetach(attacherVehicle, implementIndex)
    self:setLiftAxleState(true)
end

function AutoLiftAxle:onPostAttach(attacherVehicle, implementIndex)
    --DUmmy 
end

function AutoLiftAxle:setLiftAxleState(state, noEventSend, forceUpdate)
    if forceUpdate or self.liftAxleActive ~= state then
        self.liftAxleActive = state

        if g_server and not noEventSend then
            AutoLiftAxleEvent.sendEvent(self, state)
        end

        if self.liftAxleDesign == 1 then
            if self:getAnimationExists("Animation_LiftAxle_Front") then
                local direction = state and -1 or 1
                self:playAnimation("Animation_LiftAxle_Front", direction, self:getAnimationTime("Animation_LiftAxle_Front"), true)
            end
        elseif self.liftAxleDesign == 2 then
            if self:getAnimationExists("Animation_LiftAxle_Back") then
                local direction = state and -1 or 1
                self:playAnimation("Animation_LiftAxle_Back", direction, self:getAnimationTime("Animation_LiftAxle_Back"), true)
            end
        elseif self.liftAxleDesign == 3 then
            if self:getAnimationExists("Animation_LiftAxle_Front") then
                local direction = state and -1 or 1
                self:playAnimation("Animation_LiftAxle_Front", direction, self:getAnimationTime("Animation_LiftAxle_Front"), true)
            end
            if self:getAnimationExists("Animation_LiftAxle_Back") then
                local direction = state and -1 or 1
                self:playAnimation("Animation_LiftAxle_Back", direction, self:getAnimationTime("Animation_LiftAxle_Back"), true)
            end
        elseif self.liftAxleDesign == 4 then
            -- DUMMY
        end

        for _, wheel in ipairs(self.spec_wheels.wheels) do
            if wheel.liftAxle then
                wheel.hasGroundContact = not state
            end
        end
        self:raiseDirtyFlags(self.vehicleDirtyFlag)
    end
end

function AutoLiftAxle:onWriteStream(streamId, connection)
    streamWriteBool(streamId, self.liftAxleActive)
end

function AutoLiftAxle:onReadStream(streamId, connection)
    self.liftAxleActive = streamReadBool(streamId)
    self:setLiftAxleState(self.liftAxleActive, true, true)
end

AutoLiftAxleEvent = {}
AutoLiftAxleEvent_mt = Class(AutoLiftAxleEvent, Event)

function AutoLiftAxleEvent.emptyNew()
    local self = Event.new(AutoLiftAxleEvent_mt)
    return self
end

function AutoLiftAxleEvent.new(vehicle, state)
    local self = AutoLiftAxleEvent.emptyNew()
    self.vehicle = vehicle
    self.state = state
    return self
end

function AutoLiftAxleEvent:readStream(streamId, connection)
    self.vehicle = NetworkUtil.readNodeObject(streamId)
    self.state = streamReadBool(streamId)
    self:run(connection)
end

function AutoLiftAxleEvent:writeStream(streamId, connection)
    NetworkUtil.writeNodeObject(streamId, self.vehicle)
    streamWriteBool(streamId, self.state)
end

function AutoLiftAxleEvent:run(connection)
    if self.vehicle then
        self.vehicle:setLiftAxleState(self.state, true)
        if not connection:getIsServer() then
            g_server:broadcastEvent(self, false)
        end
    end
end

function AutoLiftAxleEvent.sendEvent(vehicle, state)
    if g_server then
        g_server:broadcastEvent(AutoLiftAxleEvent.new(vehicle, state), nil, nil, vehicle)
    else
        g_client:getServerConnection():sendEvent(AutoLiftAxleEvent.new(vehicle, state))
    end
end

InitEventClass(AutoLiftAxleEvent, "AutoLiftAxleEvent")
