-- ============================================================================
-- === AutoLiftAxleFilltype.Lua
-- === Mod my [LSMT] Modding Team 
-- === LS25 /FS25
-- === Script by [LSMT] BaTt3RiE @ 2025
-- === Ver 1.0.0.0
-- ============================================================================

AutoLiftAxleFilltype = {}

function AutoLiftAxleFilltype.prerequisitesPresent(specializations)
    return true
end

function AutoLiftAxleFilltype.registerFunctions(vehicleType)
    SpecializationUtil.registerFunction(vehicleType, "setLiftAxleState", AutoLiftAxleFilltype.setLiftAxleState)
end

function AutoLiftAxleFilltype.registerEventListeners(vehicleType)
    SpecializationUtil.registerEventListener(vehicleType, "onLoad", AutoLiftAxleFilltype)
    SpecializationUtil.registerEventListener(vehicleType, "onPostLoad", AutoLiftAxleFilltype)
    SpecializationUtil.registerEventListener(vehicleType, "onUpdate", AutoLiftAxleFilltype)
    SpecializationUtil.registerEventListener(vehicleType, "onPostDetach", AutoLiftAxleFilltype)
    SpecializationUtil.registerEventListener(vehicleType, "onPostAttach", AutoLiftAxleFilltype)
    SpecializationUtil.registerEventListener(vehicleType, "onReadStream", AutoLiftAxleFilltype)
    SpecializationUtil.registerEventListener(vehicleType, "onWriteStream", AutoLiftAxleFilltype)
end

function AutoLiftAxleFilltype:onLoad(savegame)
    self.liftAxleActive = true
    self.liftAxleCheckTimer = 0
    self.liftAxleCheckInterval = 2000
    self.shopStateApplied = false
    self.liftAxleThreshold = 10000

    self.liftAxleDesign = 3
    if self.configurations ~= nil and self.configurations["design"] ~= nil then
        self.liftAxleDesign = tonumber(self.configurations["design"]) or 3
    end
end

function AutoLiftAxleFilltype:onPostLoad(savegame)
    self:setLiftAxleState(self.liftAxleActive, false, true)
end

function AutoLiftAxleFilltype:onUpdate(dt)

    local isInShop = (g_gui.currentGuiName == "ShopConfigScreen")

    if isInShop and g_shopConfigScreen ~= nil then
        if g_shopConfigScreen.vehicle == self then
            self.liftAxleActive = true
            self.wasInShop = true

            if self.liftAxleDesign ~= 4 then
                if (self.liftAxleDesign == 1 or self.liftAxleDesign == 3)
                   and self:getAnimationExists("Animation_LiftAxle_Front") then
                    local direction = 1
                    self:playAnimation("Animation_LiftAxle_Front", direction, self:getAnimationTime("Animation_LiftAxle_Front"), true)
                end

                if (self.liftAxleDesign == 2 or self.liftAxleDesign == 3)
                   and self:getAnimationExists("Animation_LiftAxle_Back") then
                    local direction = 1
                    self:playAnimation("Animation_LiftAxle_Back", direction, self:getAnimationTime("Animation_LiftAxle_Back"), true)
                end
            end
            return
        end
    end

    if g_server then
        AutoLiftAxleFilltype.checkLiftAxleState(self, dt)
    end

end

function AutoLiftAxleFilltype.findMotorizedAttacherVehicle(vehicle)
    local currentVehicle = vehicle

    while currentVehicle ~= nil do
        if currentVehicle.getIsMotorStarted ~= nil then
            return currentVehicle
        end
        currentVehicle = currentVehicle:getAttacherVehicle()
    end

    return nil
end

function AutoLiftAxleFilltype.checkLiftAxleState(self, dt)
    local motorizedVehicle = AutoLiftAxleFilltype.findMotorizedAttacherVehicle(self)

    if motorizedVehicle == nil then
        self:setLiftAxleState(true)
        return
    end

    local motorOn = motorizedVehicle:getIsMotorStarted()
    if motorOn then
        self.liftAxleCheckTimer = self.liftAxleCheckTimer + dt
        if self.liftAxleCheckTimer >= self.liftAxleCheckInterval then
            self.liftAxleCheckTimer = 0



            local totalLoad = 0
            if self.liftAxleThreshold then
                if self.spec_fillUnits and self.spec_fillUnits.fillUnits then
                    for i, _ in ipairs(self.spec_fillUnits.fillUnits) do
                        totalLoad = totalLoad + self:getFillUnitFillLevel(i)
                    end
                else
                    totalLoad = self:getFillUnitFillLevel(1)
                end
            end

            if totalLoad > self.liftAxleThreshold then
                self:setLiftAxleState(true)
            else
                self:setLiftAxleState(false)
            end
        end
    else
        self:setLiftAxleState(true)
    end
end


function AutoLiftAxleFilltype:onPostDetach(attacherVehicle, implementIndex)
    self:setLiftAxleState(true)
end

function AutoLiftAxleFilltype:onPostAttach(attacherVehicle, implementIndex)
    --DUmmy 
end

function AutoLiftAxleFilltype:setLiftAxleState(state, noEventSend, forceUpdate)
    if forceUpdate or self.liftAxleActive ~= state then
        self.liftAxleActive = state

        if g_server and not noEventSend then
            AutoLiftAxleFilltypeEvent.sendEvent(self, state)
        end

        if self.liftAxleDesign == 1 then
            if self:getAnimationExists("Animation_LiftAxle_Front") then
                local direction = state and -1 or 1
                self:playAnimation("Animation_LiftAxle_Front", direction, self:getAnimationTime("Animation_LiftAxle_Front"), true)
            end
        elseif self.liftAxleDesign == 2 then
            if self:getAnimationExists("Animation_LiftAxle_Back") then
                local direction = state and -1 or 1
                self:playAnimation("Animation_LiftAxle_Back", direction, self:getAnimationTime("Animation_LiftAxle_Back"), true)
            end
        elseif self.liftAxleDesign == 3 then
            if self:getAnimationExists("Animation_LiftAxle_Front") then
                local direction = state and -1 or 1
                self:playAnimation("Animation_LiftAxle_Front", direction, self:getAnimationTime("Animation_LiftAxle_Front"), true)
            end
            if self:getAnimationExists("Animation_LiftAxle_Back") then
                local direction = state and -1 or 1
                self:playAnimation("Animation_LiftAxle_Back", direction, self:getAnimationTime("Animation_LiftAxle_Back"), true)
            end
        elseif self.liftAxleDesign == 4 then
            -- DUMMY
        end

        for _, wheel in ipairs(self.spec_wheels.wheels) do
            if wheel.liftAxle then
                wheel.hasGroundContact = not state
            end
        end
        self:raiseDirtyFlags(self.vehicleDirtyFlag)
    end
end

function AutoLiftAxleFilltype:onWriteStream(streamId, connection)
    streamWriteBool(streamId, self.liftAxleActive)
end

function AutoLiftAxleFilltype:onReadStream(streamId, connection)
    self.liftAxleActive = streamReadBool(streamId)
    self:setLiftAxleState(self.liftAxleActive, true, true)
end

AutoLiftAxleFilltypeEvent = {}
AutoLiftAxleFilltypeEvent_mt = Class(AutoLiftAxleFilltypeEvent, Event)

function AutoLiftAxleFilltypeEvent.emptyNew()
    local self = Event.new(AutoLiftAxleFilltypeEvent_mt)
    return self
end

function AutoLiftAxleFilltypeEvent.new(vehicle, state)
    local self = AutoLiftAxleFilltypeEvent.emptyNew()
    self.vehicle = vehicle
    self.state = state
    return self
end

function AutoLiftAxleFilltypeEvent:readStream(streamId, connection)
    self.vehicle = NetworkUtil.readNodeObject(streamId)
    self.state = streamReadBool(streamId)
    self:run(connection)
end

function AutoLiftAxleFilltypeEvent:writeStream(streamId, connection)
    NetworkUtil.writeNodeObject(streamId, self.vehicle)
    streamWriteBool(streamId, self.state)
end

function AutoLiftAxleFilltypeEvent:run(connection)
    if self.vehicle then
        self.vehicle:setLiftAxleState(self.state, true)
        if not connection:getIsServer() then
            g_server:broadcastEvent(self, false)
        end
    end
end

function AutoLiftAxleFilltypeEvent.sendEvent(vehicle, state)
    if g_server then
        g_server:broadcastEvent(AutoLiftAxleFilltypeEvent.new(vehicle, state), nil, nil, vehicle)
    else
        g_client:getServerConnection():sendEvent(AutoLiftAxleFilltypeEvent.new(vehicle, state))
    end
end

InitEventClass(AutoLiftAxleFilltypeEvent, "AutoLiftAxleFilltypeEvent")
